-- Inspired: http://www.sqlskills.com/BLOGS/PAUL/post/Inside-the-Storage-Engine-More-on-the-circular-nature-of-the-log.aspx
USE [master]
GO


DROP DATABASE LOGINFO
GO


CREATE DATABASE [LOGINFO] ON  PRIMARY 
( NAME = N'LOGINFO', 
	FILENAME = N'C:\Program Files\Microsoft SQL Server\MSSQL10_50.MSSQLSERVER\MSSQL\DATA\LOGINFO.mdf' , 
	SIZE = 51200KB , 
	MAXSIZE = UNLIMITED, 
	FILEGROWTH = 1024KB )
 LOG ON 
( NAME = N'LOGINFO_log', 
  FILENAME = N'C:\Program Files\Microsoft SQL Server\MSSQL10_50.MSSQLSERVER\MSSQL\DATA\LOGINFO_log.ldf' , 
  SIZE = 10240KB , 
  MAXSIZE = 2048GB, 
  FILEGROWTH = 10%)
GO

USE LOGINFO
GO

-- Simple Recovery Truncate In-Active VLF when CHECKPOINT is issued
ALTER DATABASE LOGINFO SET RECOVERY SIMPLE
GO

-- VLF
DBCC LOGINFO
GO


/*
Fileid - Log file id
File size - size in bytes. 
Start Offset - First 8K page header. offset starts at 8192
FSeqNo - VLF sequence number (remember T-Log is sequential and circular)
Status - 0 (Reusable or Unused) 2 (Active or Recoverable)
Parity - 0/64/128
Create LSN - 0 if VLF is added by statement
*/

SELECT * FROM fn_dblog(null,null)

-- Create table
CREATE TABLE tVLFTest(intSlno int IDENTITY (1,1), strDesc char(8000))
GO

-- INSERT 600 Records (Implicit Transaction)
SET NOCOUNT ON
INSERT INTO tVLFTest(strDesc) VALUES('test')
GO 600

-- Check VLF status
DBCC LOGINFO
GO
-- Status flag of 3 VLFs is 2 now

-- Open Explicit transaction

SET NOCOUNT ON
BEGIN TRAN
INSERT INTO tVLFTest(strDesc) VALUES('open')


-- VLF
DBCC LOGINFO
GO

-- CHECKPOINT to truncate in-active VLF
CHECKPOINT

-- VLF
DBCC LOGINFO
GO

-- INSERT 600 records (Implicit Transaction)
SET NOCOUNT ON
INSERT INTO tVLFTest(strDesc) VALUES('test')
GO 600

-- VLF
DBCC LOGINFO
GO

-- CHECKPOINT to truncate in-active VLF
CHECKPOINT

-- VLF
DBCC LOGINFO
GO
-- One transaction is still open. No VLF is cleared


-- INSERT 600 records (Implicit Transaction)
SET NOCOUNT ON
INSERT INTO tVLFTest(strDesc) VALUES('test')
GO 600

-- VLF
DBCC LOGINFO
GO
-- Log file has grown

-- Commit Open Transaction
COMMIT TRAN

-- Issue Checkpoint
CHECKPOINT

-- VLF
DBCC LOGINFO
GO

/*Shrink Log
http://support.microsoft.com/kb/907511

Typically, shrinking the transaction log file in SQL Server 2005 is faster than shrinking 
the transaction log file in SQL Server 2000. The reason is that the SQL Server 2005 log manager
creates or reuses inactive virtual log files by following the physical disk storage order. 
Therefore, the inactive part of the transaction log file is usually at the end of the file.

For example, the transaction log file may have 100 virtual log files, and only 2 virtual 
log files are used. SQL Server 2000 may store the first used virtual log file at the start 
of the transaction log file and the second used virtual log file in the middle of the 
transaction log file. To shrink the transaction log file to only 2 virtual log files, 
SQL Server fills the remaining part of the second virtual log file by using dummy log entries.
 SQL Server moves the start of the logical log to the next available virtual log file that is 
 specified by the log manager. The log manager may create a virtual log file in the middle of 
 the transaction log file just ahead of the last active virtual log file. In that case, you have to use multiple log backup operations and multiple shrink operations to successfully shrink the transaction log file to 2 virtual log files. In the worst case of this example, you may have to use 50 log backup operations and 50 shrink operations to successfully shrink the transaction log file to 2 virtual log files.

However, in SQL Server 2005, you can perform one DBCC SHRINKFILE statement to shrink the 
transaction log file immediately to 2 virtual log files. You can do this because the SQL 
Server 2005 log manager creates 2 virtual log files by following the physical disk storage
order. Both virtual log files are at the start of the transaction log file.
*/
-- http://technet.microsoft.com/en-us/library/ms189493.aspx

DBCC SHRINKFILE('LOGINFO_Log')
GO

DBCC LOGINFO
GO

-- Repeat
CHECKPOINT
GO

DBCC SHRINKFILE('LOGINFO_Log')
GO

DBCC LOGINFO
GO



-- Actual file size C:\Program Files\Microsoft SQL Server\MSSQL10_50.MSSQLSERVER\MSSQL\DATA\
