USE master
GO
DROP DATABASE RECOVERY_TEST
GO

CREATE DATABASE RECOVERY_TEST
GO
USE RECOVERY_TEST
GO
ALTER DATABASE RECOVERY_TEST SET RECOVERY SIMPLE
GO

CREATE TABLE Table1(intSlno int, strComment varchar(50))
GO

DBCC LOGINFO('RECOVERY_TEST')
GO

-- SCENARIOS:

-- TRANSACTION 1 OPEN BEFORE CHECKPOINT AND COMMITS BEFORE CHECKPOINT
-- TRANSACTION 2 OPEN BEFORE CHECKPOINT AND COMMITS AFTER CHECKPOINT
-- TRANSACTION 3 OPEN BEFORE CHECKPOINT AND KEEP IT OPEN TILL RECOVERY
-- TRANSACTION 4 OPEN AFTER CHECKPOINT AND COMMIT BEFORE RECOVERY
-- TRANSACTION 5 OPEN AFTER CHECKPOINT AND KEEP IT OPEN TILL RECOVERY 

--
-- TRANSACTION 1 OPEN BEFORE CHECKPOINT AND COMMITS BEFORE CHECKPOINT
BEGIN TRAN TRAN1
INSERT INTO Table1 VALUES(1001,'A')
COMMIT TRAN

-- TRANSACTION 2 OPEN BEFORE CHECKPOINT AND COMMITS AFTER CHECKPOINT
USE RECOVERY_TEST
GO
BEGIN TRAN TRAN2
INSERT INTO Table1 VALUES(1002,'B')
-- COMMIT THIS AFTER CHECKPOINT
COMMIT TRAN TRAN2


-- TRANSACTION 3 OPEN BEFORE CHECKPOINT AND KEEP IT OPEN TILL RECOVERY
USE RECOVERY_TEST
GO
BEGIN TRAN TRAN3
INSERT INTO Table1 VALUES(1003,'C')
-- DONT COMMIT

CHECKPOINT

-- TRANSACTION 4 OPEN AFTER CHECKPOINT AND COMMIT BEFORE RECOVERY
USE RECOVERY_TEST
GO
BEGIN TRAN TRAN4
INSERT INTO Table1 VALUES(1004,'D')
COMMIT TRAN TRAN4

-- TRANSACTION 5 OPEN AFTER CHECKPOINT AND KEEP IT OPEN TILL RECOVERY 
USE RECOVERY_TEST
GO
BEGIN TRAN TRAN5
INSERT INTO Table1 VALUES(1005,'E')

--- Restart SQL Server Service

-- Check SQL Server Log - Redo and Undo operations are performed for this database
-- 2 transactions are undone as part of Undo operation

SELECT * FROM Table1
GO
-- You can see 3 committed records
