USE MASTER
GO
DROP DATABASE EXPLORE_TLOG
GO

CREATE DATABASE EXPLORE_TLOG
GO
USE EXPLORE_TLOG
GO

dbcc loginfo
-- Start offset is 8192. T-Log has a Page header.
-- Detach db. open .ldf in Hex editor and Observe

sp_helpdb 'explore_tlog'
go

select * from sys.sysaltfiles 
where dbid = 9 and fileid = 2

-- Parameters Start LSN and End LSN
SELECT 
	[Previous LSN],			[Current LSN],		[Transaction ID],  Operation,				CONTEXT, 
	
	AllocUnitName,		 [Page ID],				[Slot ID]
	
	[Offset in Row],		[Begin Time],		 [Transaction Name],	[End Time], 
	
	[Number of Locks],		[Lock Information],  [RowLog Contents 0], 	[RowLog Contents 1], 
	
	[RowLog Contents 2],	[RowLog Contents 3], [RowLog Contents 4], 	[Log Record], 
	
	[Log Record Fixed Length], [Log Record Length]    
	
FROM fn_dblog(null,null)
GO

--
-- Observing T-Log records of CREATE TABLE
--

CREATE TABLE tEmployee(intSlno int PRIMARY KEY, strName varchar(50))
GO

-- Parameters Start LSN and End LSN
SELECT 
	[Previous LSN], 
	[Current LSN],
	Operation, 
	CONTEXT, 
	[Transaction ID], 
	AllocUnitName, 
	[Page ID], 
	[Slot ID]
	[Offset in Row], 
	[Begin Time], 
	[Transaction Name], 
	[Number of Locks],
	[Lock Information], 
	[RowLog Contents 0], 
	[RowLog Contents 1], 
	[RowLog Contents 2], 
	[RowLog Contents 3], 
	[RowLog Contents 4], 
	[Log Record]
FROM fn_dblog(null,null)
-- WHERE [Transaction ID] = '0000:0000025f'

SELECT 
	[Current LSN],
	Operation, 
	CONTEXT, 
	AllocUnitName, 
	[Page ID]
FROM fn_dblog(null,null)
WHERE [Transaction ID] = '0000:0000025f'


-- Parameters Start LSN and End LSN
SELECT 
	DISTINCT AllocUnitName 
FROM fn_dblog(null,null)
-- where [Transaction ID] = '0000:0000025f'

-- Login as DAC
-- Observe System tables in T-Log

select * from sys.sysschobjs
sp_helpindex 'sys.sysschobjs'

select * from sys.syscolpars

select * from sys.sysrowsets -- where idmajor in(2105058535)

select * from sys.sysidxstats -- where id in(2105058535)

-- back to normal connection

select * from sys.partitions -- where partition_id = 72057594038779904
select * from sys.allocation_units -- where container_id = 72057594038779904


--
-- Observing T-Log records of INSERT
--

INSERT INTO tEmployee VALUES(1001,'AAAA')
INSERT INTO tEmployee VALUES(1002,'BBBB')
GO

SELECT 
	[Previous LSN], 
	[Current LSN],
	Operation, 
	CONTEXT, 
	[Transaction ID], 
	AllocUnitName, 
	[Page ID], 
	[Slot ID]
	[Offset in Row], 
	[Begin Time], 
	[Transaction Name], 
	[Number of Locks],
	[Lock Information], 
	[RowLog Contents 0], 
	[RowLog Contents 1], 
	[RowLog Contents 2], 
	[RowLog Contents 3], 
	[RowLog Contents 4], 
	[Log Record]
FROM fn_dblog(null,null)
-- where [Transaction ID] = '0000:00000260'
GO

SELECT distinct operation FROM fn_dblog(null,null)
order by Operation


--
-- Observing T-Log records of DELTE
--

DELETE FROM tEmployee WHERE intSlno = 1002
GO

SELECT 
	[Previous LSN], 
	[Current LSN],
	Operation, 
	CONTEXT, 
	[Transaction ID], 
	AllocUnitName, 
	[Page ID], 
	[Slot ID]
	[Offset in Row], 
	[Begin Time], 
	[Transaction Name], 
	[Number of Locks],
	[Lock Information], 
	[RowLog Contents 0], 
	[RowLog Contents 1], 
	[RowLog Contents 2], 
	[RowLog Contents 3], 
	[RowLog Contents 4], 
	[Log Record]
FROM fn_dblog(null,null)
WHERE [Current LSN] >= '00000019:00000044:0001'

--
-- Observing T-Log records of DELTE AND ROLLBACK
--
BEGIN TRAN DEL_ROLLBACK
DELETE FROM tEmployee WHERE intSlno = 1001

SELECT 
	[Previous LSN], 
	[Current LSN],
	Operation, 
	CONTEXT, 
	[Transaction ID], 
	AllocUnitName, 
	[Page ID], 
	[Slot ID]
	[Offset in Row], 
	[Begin Time], 
	[Transaction Name], 
	[Number of Locks],
	[Lock Information], 
	[RowLog Contents 0], 
	[RowLog Contents 1], 
	[RowLog Contents 2], 
	[RowLog Contents 3], 
	[RowLog Contents 4], 
	[Log Record]
FROM fn_dblog(null,null)


--
-- Observing T-Log records of ROLLBACK
--

ROLLBACK
GO
SELECT 
	[Previous LSN], 
	[Current LSN],
	Operation, 
	CONTEXT, 
	[Transaction ID], 
	AllocUnitName, 
	[Page ID], 
	[Slot ID]
	[Offset in Row], 
	[Begin Time], 
	[Transaction Name], 
	[Number of Locks],
	[Lock Information], 
	[RowLog Contents 0], 
	[RowLog Contents 1], 
	[RowLog Contents 2], 
	[RowLog Contents 3], 
	[RowLog Contents 4], 
	[Log Record]
FROM fn_dblog(null,null)
GO

SELECT * FROM tEmployee
GO

--
-- Observing T-Log records of UPDATE
--

UPDATE tEmployee SET strName = 'XXYYZZ'
GO
SELECT 
	[Previous LSN], 
	[Current LSN],
	Operation, 
	CONTEXT, 
	[Transaction ID], 
	AllocUnitName, 
	[Page ID], 
	[Slot ID]
	[Offset in Row], 
	[Begin Time], 
	[Transaction Name], 
	[Number of Locks],
	[Lock Information], 
	[RowLog Contents 0], 
	[RowLog Contents 1], 
	[RowLog Contents 2], 
	[RowLog Contents 3], 
	[RowLog Contents 4], 
	[Log Record]
FROM fn_dblog(null,null)
GO

/*
0x130041414141	
0x1500585859595A5A	
0x16E9030000010000	
0x0101000C0000E7A4787D0000010200040204000AE64CB5864025	
0x	
0x00003E0019000000450000000C000200670200000000040291000000010000001B00000019000000450000000A00000100000D00000000010D000600000004000600080008001A0013004141414100001500585859595A5A16E90300000100000101000C0000E7A4787D0000010200040204000AE64CB58640250000
*/
